(defun sgml-expand-parameter-entities (&optional never-save-file)
  "Expand parameter entity references.  Offer to save file unless prefix arg."
  (interactive "P")
  (and (not never-save-file)
       (buffer-modified-p)
       buffer-file-name
       (y-or-n-p (concat "Save buffer in " buffer-file-name "? "))
       (basic-save-buffer))
  (save-excursion
    (let ((ps "\\([ \t\n]\\|--\\(-?[^-]\\)*--\\)"))
      (quick-replace "\\(%[-A-Za-z][A-Za-z0-9.]*\\)\\([^-;A-Za-z0-9.]\\)"
		     "\\1;\\2" t)
      (while
	  (re-search-forward 
	   (concat "<!ENTITY" ps "+%" ps "+\\([A-Za-z][A-Za-z0-9.-]*\\)"
		   ps "+\\(\"[^\"]*\"\\|'[^']*'\\)" ps "*>")
	   (point-max) t)
	(let ((entity (concat "%" (buffer-substring
				   (match-beginning 5)
				   (match-end 5)) ";"))
	      (replacement-string (buffer-substring
				   (1+ (match-beginning 8))
				   (1- (match-end 8)))))
	  (quick-replace entity replacement-string nil))))))

(defun quick-replace (from-string to-string regexp-flag)
  (let ((search-function (if regexp-flag 're-search-forward 'search-forward))
	(lastrepl nil))
    (save-excursion
      (while (and (not (eobp))
		  (funcall search-function from-string nil t))
	(if (eq lastrepl (point))
	    (forward-char 1)
	  (replace-match to-string t (not regexp-flag))
	  (setq lastrepl (point)))))))
